(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs'), require('rxjs/operators'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('@ng-tangram/components/notifier', ['exports', '@angular/core', 'rxjs', 'rxjs/operators', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.nt = global.nt || {}, global.nt.components = global.nt.components || {}, global.nt.components.notifier = {}), global.ng.core, global.rxjs, global.rxjs.operators, global.ng.common));
}(this, (function (exports, core, rxjs, operators, common) { 'use strict';

    var NtNotifierQueue = /** @class */ (function () {
        function NtNotifierQueue() {
            this.actionStream = new rxjs.Subject();
            this._actionQueue = [];
            this._isActionInProgress = false;
        }
        NtNotifierQueue.prototype.push = function (action) {
            this._actionQueue.push(action);
            this._tryToRunNextAction();
        };
        NtNotifierQueue.prototype.continue = function () {
            this._isActionInProgress = false;
            this._tryToRunNextAction();
        };
        NtNotifierQueue.prototype._tryToRunNextAction = function () {
            if (this._isActionInProgress || this._actionQueue.length === 0) {
                return;
            }
            this._isActionInProgress = true;
            this.actionStream.next(this._actionQueue.shift());
        };
        return NtNotifierQueue;
    }());
    NtNotifierQueue.decorators = [
        { type: core.Injectable }
    ];

    var NtNotifierRef = /** @class */ (function () {
        function NtNotifierRef(options) {
            this.template = null;
            Object.assign(this, options);
            if (options.id === undefined) {
                this.id = "ID_" + new Date().getTime();
            }
        }
        return NtNotifierRef;
    }());

    var NtNotifierConfig = /** @class */ (function () {
        function NtNotifierConfig(customOptions) {
            if (customOptions === void 0) { customOptions = {}; }
            this.animations = {
                enabled: true,
                hide: {
                    easing: 'ease',
                    offset: 50,
                    preset: 'fade',
                    speed: 300,
                },
                overlap: 150,
                shift: {
                    easing: 'ease',
                    speed: 300,
                },
                show: {
                    easing: 'ease',
                    preset: 'slide',
                    speed: 300,
                },
            };
            this.behaviour = {
                hiddenStrategy: 'content',
                hiddenTime: 2000,
                onClick: false,
                onMouseover: 'pauseAutoHide',
                showDismissButton: true,
                stacking: 4,
            };
            this.position = {
                horizontal: {
                    distance: 12,
                    position: 'middle',
                },
                vertical: {
                    distance: 12,
                    gap: 10,
                    position: 'top',
                },
            };
            if (customOptions.animations !== undefined) {
                if (customOptions.animations.enabled !== undefined) {
                    this.animations.enabled = customOptions.animations.enabled;
                }
                if (customOptions.animations.overlap !== undefined) {
                    this.animations.overlap = customOptions.animations.overlap;
                }
                if (customOptions.animations.hide !== undefined) {
                    Object.assign(this.animations.hide, customOptions.animations.hide);
                }
                if (customOptions.animations.shift !== undefined) {
                    Object.assign(this.animations.shift, customOptions.animations.shift);
                }
                if (customOptions.animations.show !== undefined) {
                    Object.assign(this.animations.show, customOptions.animations.show);
                }
            }
            if (customOptions.behaviour !== undefined) {
                Object.assign(this.behaviour, customOptions.behaviour);
            }
            if (customOptions.position !== undefined) {
                if (customOptions.position.horizontal !== undefined) {
                    Object.assign(this.position.horizontal, customOptions.position.horizontal);
                }
                if (customOptions.position.vertical !== undefined) {
                    Object.assign(this.position.vertical, customOptions.position.vertical);
                }
            }
        }
        return NtNotifierConfig;
    }());

    var NT_NOTIFIER_OPTIONS = new core.InjectionToken('nt-notifier-options');
    var NT_NOTIFIER_CONFIG = new core.InjectionToken('nt-notifier-config');

    var NtNotifier = /** @class */ (function () {
        function NtNotifier(_queueService, _config) {
            this._queueService = _queueService;
            this._config = _config;
        }
        NtNotifier.prototype.getConfig = function () {
            return this._config;
        };
        NtNotifier.prototype.show = function (notificationOptions) {
            this._queueService.push({
                payload: notificationOptions,
                type: 'SHOW',
            });
        };
        NtNotifier.prototype.hide = function (notificationId) {
            this._queueService.push({
                payload: notificationId,
                type: 'HIDE',
            });
        };
        NtNotifier.prototype.hideNewest = function () {
            this._queueService.push({
                type: 'HIDE_NEWEST',
            });
        };
        NtNotifier.prototype.hideOldest = function () {
            this._queueService.push({
                type: 'HIDE_OLDEST',
            });
        };
        NtNotifier.prototype.hideAll = function () {
            this._queueService.push({
                type: 'HIDE_ALL',
            });
        };
        NtNotifier.prototype.notify = function (type, message, notificationId) {
            var notificationOptions = { message: message, type: type };
            if (notificationId !== undefined) {
                notificationOptions.id = notificationId;
            }
            this.show(notificationOptions);
        };
        return NtNotifier;
    }());
    NtNotifier.decorators = [
        { type: core.Injectable }
    ];
    NtNotifier.ctorParameters = function () { return [
        { type: NtNotifierQueue },
        { type: NtNotifierConfig, decorators: [{ type: core.Inject, args: [NT_NOTIFIER_CONFIG,] }] }
    ]; };

    var NtNotifierContainerComponent = /** @class */ (function () {
        function NtNotifierContainerComponent(_changeDetectorRef, _notifierQueue, notifier) {
            var _this = this;
            this._changeDetectorRef = _changeDetectorRef;
            this._notifierQueue = _notifierQueue;
            this.notifications = [];
            this._config = notifier.getConfig();
            this.notifierQueueSubscription = this._notifierQueue.actionStream
                .pipe(operators.switchMap(function (action) { return _this._handleAction(action); }))
                .subscribe(function () { return _this._notifierQueue.continue(); });
        }
        NtNotifierContainerComponent.prototype.ngOnDestroy = function () {
            if (this.notifierQueueSubscription) {
                this.notifierQueueSubscription.unsubscribe();
            }
        };
        NtNotifierContainerComponent.prototype.identifyNotification = function (index, notification) {
            return notification.id;
        };
        NtNotifierContainerComponent.prototype.onNotificationDismiss = function (notificationId) {
            this._notifierQueue.push({
                payload: notificationId,
                type: 'HIDE',
            });
        };
        NtNotifierContainerComponent.prototype.onNotificationReady = function (notificationComponent) {
            var currentNotification = this.notifications[this.notifications.length - 1];
            currentNotification.component = notificationComponent;
            this._continueHandleShowAction(currentNotification);
        };
        NtNotifierContainerComponent.prototype._handleAction = function (action) {
            switch (action.type) {
                case 'SHOW':
                    return this._handleShowAction(action);
                case 'HIDE':
                    return this._handleHideAction(action);
                case 'HIDE_OLDEST':
                    return this._handleHideOldestAction(action);
                case 'HIDE_NEWEST':
                    return this._handleHideNewestAction(action);
                case 'HIDE_ALL':
                    return this._handleHideAllAction(action);
                default:
                    return new Promise(function (resolve, reject) {
                        resolve();
                    });
            }
        };
        NtNotifierContainerComponent.prototype._handleShowAction = function (action) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                _this._tempPromiseResolver = resolve;
                _this.addNotificationToList(new NtNotifierRef(action.payload));
            });
        };
        NtNotifierContainerComponent.prototype._continueHandleShowAction = function (notification) {
            var _this = this;
            var numberOfNotifications = this.notifications.length;
            if (numberOfNotifications === 1) {
                notification.component.show().then(this._tempPromiseResolver);
            }
            else {
                var implicitStackingLimit = 2;
                if (this._config.behaviour.stacking === false ||
                    this._config.behaviour.stacking < implicitStackingLimit) {
                    this.notifications[0].component.hide().then(function () {
                        _this.removeNotificationFromList(_this.notifications[0]);
                        notification.component.show().then(_this._tempPromiseResolver);
                    });
                }
                else {
                    var stepPromises_1 = [];
                    if (numberOfNotifications > this._config.behaviour.stacking) {
                        var oldNotifications_1 = this.notifications.slice(1, numberOfNotifications - 1);
                        if (this._config.animations.enabled) {
                            if (this._config.animations.overlap !== false &&
                                this._config.animations.overlap > 0) {
                                stepPromises_1.push(this.notifications[0].component.hide());
                                setTimeout(function () {
                                    stepPromises_1.push(_this.shiftNotifications(oldNotifications_1, notification.component.getHeight(), true));
                                }, this._config.animations.hide.speed - this._config.animations.overlap);
                                setTimeout(function () {
                                    stepPromises_1.push(notification.component.show());
                                }, this._config.animations.hide.speed + this._config.animations.shift.speed - this._config.animations.overlap);
                            }
                            else {
                                stepPromises_1.push(new Promise(function (resolve, reject) {
                                    _this.notifications[0].component.hide().then(function () {
                                        _this.shiftNotifications(oldNotifications_1, notification.component.getHeight(), true).then(function () {
                                            notification.component.show().then(resolve);
                                        });
                                    });
                                }));
                            }
                        }
                        else {
                            stepPromises_1.push(this.notifications[0].component.hide());
                            stepPromises_1.push(this.shiftNotifications(oldNotifications_1, notification.component.getHeight(), true));
                            stepPromises_1.push(notification.component.show());
                        }
                    }
                    else {
                        var oldNotifications_2 = this.notifications.slice(0, numberOfNotifications - 1);
                        if (this._config.animations.enabled) {
                            if (this._config.animations.overlap !== false &&
                                this._config.animations.overlap > 0) {
                                stepPromises_1.push(this.shiftNotifications(oldNotifications_2, notification.component.getHeight(), true));
                                setTimeout(function () {
                                    stepPromises_1.push(notification.component.show());
                                }, this._config.animations.shift.speed - this._config.animations.overlap);
                            }
                            else {
                                stepPromises_1.push(new Promise(function (resolve, reject) {
                                    _this.shiftNotifications(oldNotifications_2, notification.component.getHeight(), true).then(function () {
                                        notification.component.show().then(resolve);
                                    });
                                }));
                            }
                        }
                        else {
                            stepPromises_1.push(this.shiftNotifications(oldNotifications_2, notification.component.getHeight(), true));
                            stepPromises_1.push(notification.component.show());
                        }
                    }
                    Promise.all(stepPromises_1).then(function () {
                        if (numberOfNotifications > _this._config.behaviour.stacking) {
                            _this.removeNotificationFromList(_this.notifications[0]);
                        }
                        _this._tempPromiseResolver();
                    });
                }
            }
        };
        NtNotifierContainerComponent.prototype._handleHideAction = function (action) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                var stepPromises = [];
                var notification = _this.findNotificationById(action.payload);
                if (notification === undefined) {
                    resolve();
                    return;
                }
                var notificationIndex = _this.findNotificationIndexById(action.payload);
                if (notificationIndex === undefined) {
                    resolve();
                    return;
                }
                var oldNotifications = _this.notifications.slice(0, notificationIndex);
                if (oldNotifications.length > 0) {
                    if (_this._config.animations.enabled &&
                        _this._config.animations.hide.speed > 0) {
                        if (_this._config.animations.overlap !== false &&
                            _this._config.animations.overlap > 0) {
                            stepPromises.push(notification.component.hide());
                            setTimeout(function () {
                                stepPromises.push(_this.shiftNotifications(oldNotifications, notification.component.getHeight(), false));
                            }, _this._config.animations.hide.speed - _this._config.animations.overlap);
                        }
                        else {
                            notification.component.hide().then(function () {
                                stepPromises.push(_this.shiftNotifications(oldNotifications, notification.component.getHeight(), false));
                            });
                        }
                    }
                    else {
                        stepPromises.push(notification.component.hide());
                        stepPromises.push(_this.shiftNotifications(oldNotifications, notification.component.getHeight(), false));
                    }
                }
                else {
                    stepPromises.push(notification.component.hide());
                }
                Promise.all(stepPromises).then(function () {
                    _this.removeNotificationFromList(notification);
                    resolve();
                });
            });
        };
        NtNotifierContainerComponent.prototype._handleHideOldestAction = function (action) {
            if (this.notifications.length === 0) {
                return new Promise(function (resolve, reject) {
                    resolve();
                });
            }
            else {
                action.payload = this.notifications[0].id;
                return this._handleHideAction(action);
            }
        };
        NtNotifierContainerComponent.prototype._handleHideNewestAction = function (action) {
            if (this.notifications.length === 0) {
                return new Promise(function (resolve, reject) {
                    resolve();
                });
            }
            else {
                action.payload = this.notifications[this.notifications.length - 1].id;
                return this._handleHideAction(action);
            }
        };
        NtNotifierContainerComponent.prototype._handleHideAllAction = function (action) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                var numberOfNotifications = _this.notifications.length;
                if (numberOfNotifications === 0) {
                    resolve();
                    return;
                }
                if (_this._config.animations.enabled &&
                    _this._config.animations.hide.speed > 0 &&
                    _this._config.animations.hide.offset !== false &&
                    _this._config.animations.hide.offset > 0) {
                    var _loop_1 = function (i) {
                        var animationOffset = _this._config.position.vertical.position === 'top'
                            ? numberOfNotifications - 1
                            : i;
                        setTimeout(function () {
                            _this.notifications[i].component.hide().then(function () {
                                if ((_this._config.position.vertical.position === 'top' && i === 0) ||
                                    (_this._config.position.vertical.position === 'bottom' &&
                                        i === numberOfNotifications - 1)) {
                                    _this.removeAllNotificationsFromList();
                                    resolve();
                                }
                            });
                        }, _this._config.animations.hide.offset * animationOffset);
                    };
                    for (var i = numberOfNotifications - 1; i >= 0; i--) {
                        _loop_1(i);
                    }
                }
                else {
                    var stepPromises = [];
                    for (var i = numberOfNotifications - 1; i >= 0; i--) {
                        stepPromises.push(_this.notifications[i].component.hide());
                    }
                    Promise.all(stepPromises).then(function () {
                        _this.removeAllNotificationsFromList();
                        resolve();
                    });
                }
            });
        };
        NtNotifierContainerComponent.prototype.shiftNotifications = function (notifications, distance, toMakePlace) {
            return new Promise(function (resolve, reject) {
                if (notifications.length === 0) {
                    resolve();
                    return;
                }
                var notificationPromises = [];
                for (var i = notifications.length - 1; i >= 0; i--) {
                    notificationPromises.push(notifications[i].component.shift(distance, toMakePlace));
                }
                Promise.all(notificationPromises).then(resolve);
            });
        };
        NtNotifierContainerComponent.prototype.addNotificationToList = function (notification) {
            this.notifications.push(notification);
            this._changeDetectorRef.markForCheck();
        };
        NtNotifierContainerComponent.prototype.removeNotificationFromList = function (notification) {
            this.notifications = this.notifications.filter(function (item) { return item.component !== notification.component; });
            this._changeDetectorRef.markForCheck();
        };
        NtNotifierContainerComponent.prototype.removeAllNotificationsFromList = function () {
            this.notifications = [];
            this._changeDetectorRef.markForCheck();
        };
        NtNotifierContainerComponent.prototype.findNotificationById = function (notificationId) {
            return this.notifications.find(function (currentNotification) { return currentNotification.id === notificationId; });
        };
        NtNotifierContainerComponent.prototype.findNotificationIndexById = function (notificationId) {
            var notificationIndex = this.notifications.findIndex(function (currentNotification) { return currentNotification.id === notificationId; });
            return notificationIndex !== -1 ? notificationIndex : undefined;
        };
        return NtNotifierContainerComponent;
    }());
    NtNotifierContainerComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'nt-notifier-container',
                    template: "\n    <nt-notifier\n      *ngFor=\"let notification of notifications; trackBy: identifyNotification\"\n      [notification]=\"notification\"\n      (ready)=\"onNotificationReady($event)\"\n      (dismiss)=\"onNotificationDismiss($event)\">\n    </nt-notifier>\n  ",
                    encapsulation: core.ViewEncapsulation.None,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    host: {
                        class: 'nt-notifier-container',
                    }
                },] }
    ];
    NtNotifierContainerComponent.ctorParameters = function () { return [
        { type: core.ChangeDetectorRef },
        { type: NtNotifierQueue },
        { type: NtNotifier }
    ]; };

    var ɵ0 = function () {
        return {
            from: { opacity: '1', },
            to: { opacity: '0', },
        };
    }, ɵ1 = function () {
        return {
            from: { opacity: '0', },
            to: { opacity: '1', },
        };
    };
    var fade = {
        hide: ɵ0,
        show: ɵ1,
    };

    var ɵ0$1 = function (notification) {
        var config = notification.component.getConfig();
        var shift = notification.component.getShift();
        var from;
        var to;
        if (config.position.horizontal.position === 'left') {
            from = {
                transform: "translate3d( 0, " + shift + "px, 0 )",
            };
            to = {
                transform: "translate3d( calc( -100% - " + config.position.horizontal.distance + "px - 10px ), " + shift + "px, 0 )",
            };
        }
        else if (config.position.horizontal.position === 'right') {
            from = {
                transform: "translate3d( 0, " + shift + "px, 0 )",
            };
            to = {
                transform: "translate3d( calc( 100% + " + config.position.horizontal.distance + "px + 10px ), " + shift + "px, 0 )",
            };
        }
        else {
            var horizontalPosition = void 0;
            if (config.position.vertical.position === 'top') {
                horizontalPosition = "calc( -100% - " + config.position.horizontal.distance + "px - 10px )";
            }
            else {
                horizontalPosition = "calc( 100% + " + config.position.horizontal.distance + "px + 10px )";
            }
            from = {
                transform: "translate3d( -50%, " + shift + "px, 0 )",
            };
            to = {
                transform: "translate3d( -50%, " + horizontalPosition + ", 0 )",
            };
        }
        return {
            from: from,
            to: to,
        };
    }, ɵ1$1 = function (notification) {
        var config = notification.component.getConfig();
        var from;
        var to;
        if (config.position.horizontal.position === 'left') {
            from = {
                transform: "translate3d( calc( -100% - " + config.position.horizontal.distance + "px - 10px ), 0, 0 )",
            };
            to = {
                transform: 'translate3d( 0, 0, 0 )',
            };
        }
        else if (config.position.horizontal.position === 'right') {
            from = {
                transform: "translate3d( calc( 100% + " + config.position.horizontal.distance + "px + 10px ), 0, 0 )",
            };
            to = {
                transform: 'translate3d( 0, 0, 0 )',
            };
        }
        else {
            var horizontalPosition = void 0;
            if (config.position.vertical.position === 'top') {
                horizontalPosition = "calc( -100% - " + config.position.horizontal.distance + "px - 10px )";
            }
            else {
                horizontalPosition = "calc( 100% + " + config.position.horizontal.distance + "px + 10px )";
            }
            from = {
                transform: "translate3d( -50%, " + horizontalPosition + ", 0 )",
            };
            to = {
                transform: 'translate3d( -50%, 0, 0 )',
            };
        }
        return {
            from: from,
            to: to,
        };
    };
    var slide = {
        hide: ɵ0$1,
        show: ɵ1$1,
    };

    var NtNotifierAnimation = /** @class */ (function () {
        function NtNotifierAnimation() {
            this._animationPresets = { fade: fade, slide: slide };
        }
        NtNotifierAnimation.prototype.getAnimationData = function (direction, notification) {
            var keyframes;
            var duration;
            var easing;
            if (direction === 'show') {
                keyframes = this._animationPresets[notification.component.getConfig().animations.show.preset].show(notification);
                duration = notification.component.getConfig().animations.show.speed;
                easing = notification.component.getConfig().animations.show.easing;
            }
            else {
                keyframes = this._animationPresets[notification.component.getConfig().animations.hide.preset].hide(notification);
                duration = notification.component.getConfig().animations.hide.speed;
                easing = notification.component.getConfig().animations.hide.easing;
            }
            return {
                keyframes: [
                    keyframes.from,
                    keyframes.to
                ],
                options: {
                    duration: duration,
                    easing: easing,
                    fill: 'forwards'
                }
            };
        };
        return NtNotifierAnimation;
    }());
    NtNotifierAnimation.decorators = [
        { type: core.Injectable }
    ];

    var DEFAULT_NOTIFIER_ICONS = {
        info: 'fas fa-info-circle',
        success: 'far fa-check-circle',
        warning: 'fas fa-exclamation-circle',
        error: 'far fa-times-circle',
        close: 'fas fa-times'
    };
    var NT_NOTIFIER_ICONS = new core.InjectionToken('nt-notifier-icons');

    var NtNotifierTimer = /** @class */ (function () {
        function NtNotifierTimer() {
            this._now = 0;
            this._remaining = 0;
        }
        NtNotifierTimer.prototype.start = function (duration) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                _this._remaining = duration;
                _this._finishPromiseResolver = resolve;
                _this.continue();
            });
        };
        NtNotifierTimer.prototype.pause = function () {
            clearTimeout(this._timerId);
            this._remaining -= new Date().getTime() - this._now;
        };
        NtNotifierTimer.prototype.continue = function () {
            var _this = this;
            this._now = new Date().getTime();
            this._timerId = window.setTimeout(function () {
                _this._finish();
            }, this._remaining);
        };
        NtNotifierTimer.prototype.stop = function () {
            clearTimeout(this._timerId);
            this._remaining = 0;
        };
        NtNotifierTimer.prototype._finish = function () {
            this._finishPromiseResolver();
        };
        return NtNotifierTimer;
    }());
    NtNotifierTimer.decorators = [
        { type: core.Injectable }
    ];

    var TYPE_COLORS_MAP = {
        default: 'secondary',
        info: 'primary',
        success: 'success',
        warning: 'warning',
        error: 'alert',
    };
    var NtNotifierComponent = /** @class */ (function () {
        function NtNotifierComponent(elementRef, notifier, renderer, _notifierTimer, _notifierAnimation, icons) {
            this.renderer = renderer;
            this._notifierTimer = _notifierTimer;
            this._notifierAnimation = _notifierAnimation;
            this.icons = icons;
            this.ready = new core.EventEmitter();
            this.dismiss = new core.EventEmitter();
            this._elementShift = 0;
            this.config = notifier.getConfig();
            this._element = elementRef.nativeElement;
            this.icons = Object.assign(Object.assign({}, DEFAULT_NOTIFIER_ICONS), icons);
        }
        Object.defineProperty(NtNotifierComponent.prototype, "_hasIcon", {
            get: function () { return this.notification.type !== 'default'; },
            enumerable: false,
            configurable: true
        });
        NtNotifierComponent.prototype.ngAfterViewInit = function () {
            this._setup();
            this._elementHeight = this._element.offsetHeight;
            this._elementWidth = this._element.offsetWidth;
            this.ready.emit(this);
        };
        NtNotifierComponent.prototype.getConfig = function () {
            return this.config;
        };
        NtNotifierComponent.prototype.getHeight = function () {
            return this._elementHeight;
        };
        NtNotifierComponent.prototype.getWidth = function () {
            return this._elementWidth;
        };
        NtNotifierComponent.prototype.getShift = function () {
            return this._elementShift;
        };
        NtNotifierComponent.prototype.show = function () {
            var _this = this;
            return new Promise(function (resolve, reject) {
                if (_this.config.animations.enabled && _this.config.animations.show.speed > 0) {
                    var animationData = _this._notifierAnimation.getAnimationData('show', _this.notification);
                    var animatedProperties = Object.keys(animationData.keyframes[0]);
                    for (var i = animatedProperties.length - 1; i >= 0; i--) {
                        _this.renderer.setStyle(_this._element, animatedProperties[i], animationData.keyframes[0][animatedProperties[i]]);
                    }
                    _this.renderer.setStyle(_this._element, 'visibility', 'visible');
                    var animation = _this._element.animate(animationData.keyframes, animationData.options);
                    animation.onfinish = function () {
                        _this._startHideTimer();
                        resolve();
                    };
                }
                else {
                    _this.renderer.setStyle(_this._element, 'visibility', 'visible');
                    _this._startHideTimer();
                    resolve();
                }
            });
        };
        NtNotifierComponent.prototype.hide = function () {
            var _this = this;
            return new Promise(function (resolve, reject) {
                _this._stopHideTimer();
                if (_this.config.animations.enabled &&
                    _this.config.animations.hide.speed > 0) {
                    var animationData = _this._notifierAnimation.getAnimationData('hide', _this.notification);
                    var animation = _this._element.animate(animationData.keyframes, animationData.options);
                    animation.onfinish = function () {
                        resolve();
                    };
                }
                else {
                    resolve();
                }
            });
        };
        NtNotifierComponent.prototype.shift = function (distance, shiftToMakePlace) {
            var _this = this;
            return new Promise(function (resolve, reject) {
                var newElementShift;
                if ((_this.config.position.vertical.position === 'top' &&
                    shiftToMakePlace) ||
                    (_this.config.position.vertical.position === 'bottom' &&
                        !shiftToMakePlace)) {
                    newElementShift =
                        _this._elementShift + distance + _this.config.position.vertical.gap;
                }
                else {
                    newElementShift =
                        _this._elementShift - distance - _this.config.position.vertical.gap;
                }
                var horizontalPosition = _this.config.position.horizontal.position === 'middle' ? '-50%' : '0';
                if (_this.config.animations.enabled &&
                    _this.config.animations.shift.speed > 0) {
                    var animationData = {
                        keyframes: [
                            {
                                transform: "translate3d( " + horizontalPosition + ", " + _this._elementShift + "px, 0 )",
                            },
                            {
                                transform: "translate3d( " + horizontalPosition + ", " + newElementShift + "px, 0 )",
                            },
                        ],
                        options: {
                            duration: _this.config.animations.shift.speed,
                            easing: _this.config.animations.shift.easing,
                            fill: 'forwards',
                        },
                    };
                    _this._elementShift = newElementShift;
                    var animation = _this._element.animate(animationData.keyframes, animationData.options);
                    animation.onfinish = function () {
                        resolve();
                    };
                }
                else {
                    _this.renderer.setStyle(_this._element, 'transform', "translate3d( " + horizontalPosition + ", " + newElementShift + "px, 0 )");
                    _this._elementShift = newElementShift;
                    resolve();
                }
            });
        };
        NtNotifierComponent.prototype.onClickDismiss = function () {
            this.dismiss.emit(this.notification.id);
        };
        NtNotifierComponent.prototype.onNotificationMouseover = function () {
            if (this.config.behaviour.onMouseover === 'pauseAutoHide') {
                this._pauseHideTimer();
            }
            else if (this.config.behaviour.onMouseover === 'resetAutoHide') {
                this._stopHideTimer();
            }
        };
        NtNotifierComponent.prototype.onNotificationMouseout = function () {
            if (this.config.behaviour.onMouseover === 'pauseAutoHide') {
                this._continueHideTimer();
            }
            else if (this.config.behaviour.onMouseover === 'resetAutoHide') {
                this._startHideTimer();
            }
        };
        NtNotifierComponent.prototype.onNotificationClick = function () {
            if (this.config.behaviour.onClick === 'hide') {
                this.onClickDismiss();
            }
        };
        NtNotifierComponent.prototype._startHideTimer = function () {
            var _this = this;
            if (this._getAutoHideable()) {
                this._notifierTimer.start(this._getHiddenTime()).then(function () {
                    _this.onClickDismiss();
                });
            }
        };
        NtNotifierComponent.prototype._pauseHideTimer = function () {
            if (this._getAutoHideable()) {
                this._notifierTimer.pause();
            }
        };
        NtNotifierComponent.prototype._continueHideTimer = function () {
            if (this._getAutoHideable()) {
                this._notifierTimer.continue();
            }
        };
        NtNotifierComponent.prototype._stopHideTimer = function () {
            if (this._getAutoHideable()) {
                this._notifierTimer.stop();
            }
        };
        NtNotifierComponent.prototype._getAutoHideable = function () {
            return this.config.behaviour.hiddenStrategy === 'content' ||
                this.config.behaviour.hiddenStrategy === 'absoluted';
        };
        NtNotifierComponent.prototype._getHiddenTime = function () {
            var _a;
            var time = Math.max(((_a = this.config.behaviour) === null || _a === void 0 ? void 0 : _a.hiddenTime) || 0, 0);
            if (this.config.behaviour.hiddenStrategy === 'content') {
                time += (this.notification.message || '').length * 100;
            }
            return time;
        };
        NtNotifierComponent.prototype._setup = function () {
            if (this.config.position.horizontal.position === 'left') {
                this.renderer.setStyle(this._element, 'left', this.config.position.horizontal.distance + "px");
            }
            else if (this.config.position.horizontal.position === 'right') {
                this.renderer.setStyle(this._element, 'right', this.config.position.horizontal.distance + "px");
            }
            else {
                this.renderer.setStyle(this._element, 'left', '50%');
                this.renderer.setStyle(this._element, 'transform', 'translate3d( -50%, 0, 0 )');
            }
            if (this.config.position.vertical.position === 'top') {
                this.renderer.setStyle(this._element, 'top', this.config.position.vertical.distance + "px");
            }
            else {
                this.renderer.setStyle(this._element, 'bottom', this.config.position.vertical.distance + "px");
            }
            this.renderer.addClass(this._element, TYPE_COLORS_MAP[this.notification.type]);
        };
        return NtNotifierComponent;
    }());
    NtNotifierComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'nt-notifier',
                    template: "<ng-container *ngIf=\"notification.template; else predefinedNotification\" [ngTemplateOutlet]=\"notification.template\" [ngTemplateOutletContext]=\"{ notification: notification }\">\n</ng-container>\n\n<ng-template #predefinedNotification>\n  <div class=\"nt-notifier-icon\" *ngIf=\"_hasIcon\">\n    <span [class]=\"icons[notification.type]\"></span>\n  </div>\n\t<div class=\"nt-notifier-message\">\n    {{ notification.message }}\n  </div>\n\t<button class=\"nt-notifier-button\" type=\"button\" title=\"dismiss\" *ngIf=\"config.behaviour.showDismissButton\" (click)=\"onClickDismiss()\">\n\t\t<span [class]=\"icons.close\"></span>\n\t</button>\n</ng-template>\n",
                    encapsulation: core.ViewEncapsulation.None,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    providers: [NtNotifierTimer],
                    host: {
                        'class': 'nt-notifier',
                        '(click)': 'onNotificationClick()',
                        '(mouseout)': 'onNotificationMouseout()',
                        '(mouseover)': 'onNotificationMouseover()'
                    }
                },] }
    ];
    NtNotifierComponent.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: NtNotifier },
        { type: core.Renderer2 },
        { type: NtNotifierTimer },
        { type: NtNotifierAnimation },
        { type: undefined, decorators: [{ type: core.Optional }, { type: core.Inject, args: [NT_NOTIFIER_ICONS,] }] }
    ]; };
    NtNotifierComponent.propDecorators = {
        notification: [{ type: core.Input }],
        ready: [{ type: core.Output }],
        dismiss: [{ type: core.Output }]
    };

    function notifierCustomConfigFactory(options) {
        return new NtNotifierConfig(options);
    }
    function notifierDefaultConfigFactory() {
        return new NtNotifierConfig({});
    }
    var NtNotifierModule = /** @class */ (function () {
        function NtNotifierModule() {
        }
        NtNotifierModule.withConfig = function (options) {
            if (options === void 0) { options = {}; }
            return {
                ngModule: NtNotifierModule,
                providers: [
                    {
                        provide: NT_NOTIFIER_OPTIONS,
                        useValue: options,
                    },
                    {
                        deps: [NT_NOTIFIER_OPTIONS],
                        provide: NT_NOTIFIER_CONFIG,
                        useFactory: notifierCustomConfigFactory,
                    },
                ],
            };
        };
        return NtNotifierModule;
    }());
    NtNotifierModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [NtNotifierContainerComponent, NtNotifierComponent],
                    exports: [NtNotifierContainerComponent],
                    imports: [common.CommonModule],
                    providers: [
                        NtNotifierAnimation,
                        NtNotifier,
                        NtNotifierQueue,
                        { provide: NT_NOTIFIER_CONFIG, useFactory: notifierDefaultConfigFactory },
                    ],
                },] }
    ];

    exports.NT_NOTIFIER_CONFIG = NT_NOTIFIER_CONFIG;
    exports.NT_NOTIFIER_OPTIONS = NT_NOTIFIER_OPTIONS;
    exports.NtNotifier = NtNotifier;
    exports.NtNotifierComponent = NtNotifierComponent;
    exports.NtNotifierConfig = NtNotifierConfig;
    exports.NtNotifierContainerComponent = NtNotifierContainerComponent;
    exports.NtNotifierModule = NtNotifierModule;
    exports.notifierCustomConfigFactory = notifierCustomConfigFactory;
    exports.notifierDefaultConfigFactory = notifierDefaultConfigFactory;
    exports.ɵa = NtNotifierQueue;
    exports.ɵb = NtNotifierTimer;
    exports.ɵd = NT_NOTIFIER_ICONS;
    exports.ɵe = NtNotifierAnimation;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=notifier.umd.js.map
